@@ -6,7 +6,7 @@ Mandel - Async model layer for MongoDB objects using Mango
 
 =head1 VERSION
 
-0.28
+0.29
 
 =head1 SYNOPSIS
 
@@ -104,7 +104,7 @@ use Mandel::Model;
 use Mango;
 use Carp 'confess';
 
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 =head1 ATTRIBUTES
 
@@ -181,6 +181,8 @@ sub all_document_names {
 
   for my $ns (@{$self->namespaces}) {
     for my $name (find_modules $ns) {
+      my $e = load_class $name;
+      next unless $name->isa('Mandel::Document');
       $name =~ s/^${ns}:://;
       $names{Mojo::Util::decamelize($name)} = 1;
     }
